// Importing necessary D3 modules
import {
  select,
  csv,
  scaleBand,
  scaleLinear,
  axisBottom,
  axisLeft,
  extent,
  line,
  curveBasis,
} from 'd3';

// Selecting the SVG element from the DOM
const svg = select('svg');

// Getting the width and height of the SVG element
const width = +svg.attr('width');
const height = +svg.attr('height');

// Render function to create the line chart
const render = (data) => {
  // Defining the accessor functions for x and y values
  const xValue = (d) => d.model;
  const yValue = (d) => d.mpg;

  // Labels for axes
  const xAxisLabel = 'Car Model';
  const yAxisLabel = 'Miles Per Gallon';

  // Margin for the chart
  const margin = {
    top: 60,
    right: 40,
    bottom: 88,
    left: 150,
  };

  // Calculating inner width and height of the chart
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  // Creating scales for x and y axes
  const xScale = scaleBand()
    .domain(data.map(xValue))
    .range([0, innerWidth])
    .padding(0.1);

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();

  // Appending a group element to the SVG for the chart
  const g = svg
    .append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);

  // Creating x and y axes
  const xAxis = axisBottom(xScale);
  const yAxis = axisLeft(yScale).tickFormat(
    (d) => `${d} mpg`,
  );

  // Appending y-axis to the chart
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();

  // Adding y-axis label
  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -70)
    .attr('x', -innerHeight / 2)
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);

  // Appending x-axis to the chart
  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);

  // Rotating x-axis labels for better readability
  xAxisG
    .selectAll('.tick text')
    .attr('transform', 'rotate(-45)')
    .style('text-anchor', 'end');

  // Adding x-axis label
  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 50)
    .attr('x', innerWidth / 2)
    .text(xAxisLabel);

  // Line generator function for the line chart
  const lineGenerator = line()
    .x((d) => xScale(xValue(d)) + xScale.bandwidth() / 2)
    .y((d) => yScale(yValue(d)))
    .curve(curveBasis);

  // Adding the line to the chart
  g.append('path')
    .attr('class', 'line-path')
    .attr('d', lineGenerator(data));
};

// URL of the dataset
const csvUrl =
  'https://gist.githubusercontent.com/NandithaSura/784eef9bc929bc7f7eae9627b8fd74d4/raw/2654abeb912711ee2a3741ecad4d03240fa850bb/ice7.csv';

// Load the data and call the render function
csv(csvUrl).then((data) => {
  render(data);
});
